from typing import Annotated

from unpage.cli.mcp.tools._app import tools_app
from unpage.cli.options import DEFAULT_PROFILE, ProfileParameter
from unpage.config import load_config
from unpage.config.utils import get_config_dir
from unpage.knowledge import Graph
from unpage.mcp import Context, build_mcp_server
from unpage.plugins import PluginManager
from unpage.telemetry import client as telemetry
from unpage.telemetry import prepare_profile_for_telemetry


@tools_app.command(name="list")
async def list_tools(
    *,
    profile: Annotated[str, ProfileParameter] = DEFAULT_PROFILE,
) -> None:
    """List all MCP tools available from enabled plugins.

    Parameters
    ----------
    profile
        The profile to use
    """
    await telemetry.send_event(
        {
            "command": "mcp tools list",
            **prepare_profile_for_telemetry(profile),
        }
    )
    config = load_config(profile)
    plugins = PluginManager(config=config)
    context = Context(
        profile=profile,
        config=config,
        plugins=plugins,
        graph=Graph(get_config_dir(profile) / "graph.json"),
    )
    mcp = await build_mcp_server(context)

    tools = await mcp.get_tools()
    for key, tool in tools.items():
        cmd = [key]
        for arg, arg_data in tool.parameters["properties"].items():
            arg_type = arg_data.get("type", "unknown")
            if "anyOf" in arg_data:
                arg_type = "|".join(t["type"] for t in arg_data["anyOf"])
            cmd.append(f"<{arg}:{arg_type}>")
        print(" ".join(cmd))



import inspect

import rich

from unpage.cli._app import app
from unpage.telemetry import client as telemetry


@app.command
async def version(*, json: bool = False) -> None:
    """Display the version of the Unpage CLI.

    Parameters
    ----------
    json
        Return the version information as JSON
    """
    await telemetry.send_event(
        {
            "command": "version",
            "json_output": json,
        }
    )

    from dspy import __version__ as dspy_version
    from dspy.adapters.types.tool import Tool

    from unpage import __version__

    dspy_parse_function_tool_source = inspect.getsource(Tool._parse_function)
    dspy_tool_no_input_args_bugfix_present = all(
        " is not None else " in line for line in dspy_parse_function_tool_source.splitlines()[-3:-1]
    )

    if json:
        rich.print_json(
            data={
                "unpage": __version__,
                "dspy": dspy_version,
                "dspy_tool_no_input_args_bugfix_present": dspy_tool_no_input_args_bugfix_present,
            }
        )
        return
    print(f"unpage {__version__} (dspy {dspy_version} {dspy_tool_no_input_args_bugfix_present=})")



import sys
from typing import Annotated

from rich import print

from unpage.agent.analysis import AnalysisAgent
from unpage.cli.agent._app import agent_app
from unpage.cli.options import DEFAULT_PROFILE, ProfileParameter
from unpage.telemetry import client as telemetry
from unpage.telemetry import prepare_profile_for_telemetry


@agent_app.command
async def route(
    payload: str | None = None,
    /,
    *,
    profile: Annotated[str, ProfileParameter] = DEFAULT_PROFILE,
    debug: bool = False,
) -> None:
    """Determine which agent will be used to analyze the given payload.

    A payload can be passed as an argument or piped to stdin.

    Parameters
    ----------
    payload
        The alert payload to analyze. Alternatively, you can pipe the payload to stdin.
    profile
        The profile to use
    debug
        Enable debug mode to print the history of the routing agent.
    """
    await telemetry.send_event(
        {
            "command": "agent route",
            **prepare_profile_for_telemetry(profile),
            "debug": debug,
        }
    )
    # Read data from stdin if it's being piped to us.
    if not sys.stdin.isatty():
        if payload is not None:
            print("[red]Cannot pass a payload argument when piping data to stdin.[/red]")
            sys.exit(1)
        data = sys.stdin.read().strip()
    else:
        # Otherwise, use the payload argument.
        data = payload

    if not data:
        print("[red]No payload provided.[/red]")
        print(
            "[bold]Pass an alert payload as an argument or pipe the payload data to stdin.[/bold]"
        )
        sys.exit(1)

    # Run the analysis with the specific agent
    try:
        analysis_agent = AnalysisAgent(profile)
        result = await analysis_agent.acall(payload=data, route_only=True)
        if debug:
            print("\n\n===== DEBUG OUTPUT =====\n")
            analysis_agent.inspect_history(n=1000)
            print("\n===== END DEBUG OUTPUT =====\n\n")
        print(result)
    except Exception as ex:
        print(f"[red]Analysis failed:[/red] {ex}")
        sys.exit(1)



from cyclopts import App, Parameter

from unpage.cli.agent._app import agent_app
from unpage.cli.graph._app import graph_app
from unpage.cli.mcp._app import mcp_app
from unpage.cli.mlflow._app import mlflow_app
from unpage.warnings import filter_all_warnings

filter_all_warnings()

app = App(
    default_parameter=Parameter(
        # Disable automatic creation of "negative" options (e.g. --no-foo)
        negative=()
    )
)

app.command(agent_app, name="agent")
app.command(graph_app, name="graph")
app.command(mcp_app, name="mcp")
app.command(mlflow_app, name="mlflow")



import sys
from typing import TYPE_CHECKING, Annotated, cast

from rich import print

from unpage.agent.analysis import AnalysisAgent
from unpage.agent.utils import load_agent
from unpage.cli.agent._app import agent_app
from unpage.cli.options import DEFAULT_PROFILE, ProfileParameter
from unpage.config.utils import load_config
from unpage.plugins.base import PluginManager
from unpage.telemetry import client as telemetry
from unpage.telemetry import hash_value, prepare_profile_for_telemetry

if TYPE_CHECKING:
    from unpage.plugins.pagerduty.plugin import PagerDutyPlugin


@agent_app.command
async def run(
    agent_name: str,
    payload: str | None = None,
    /,
    *,
    pagerduty_incident: str | None = None,
    profile: Annotated[str, ProfileParameter] = DEFAULT_PROFILE,
    debug: bool = False,
) -> None:
    """Run an agent with the provided payload and print the analysis.

    A payload can be passed as an argument or piped to stdin.

    Parameters
    ----------
    agent_name
        The name of the agent to run
    payload
        The alert payload to analyze. Alternatively, you can pipe the payload to stdin.
    pagerduty_incident
        PagerDuty incident ID or URL to use instead of payload or stdin
    profile
        The profile to use
    debug
        Enable debug mode to print the history of the agent
    """
    await telemetry.send_event(
        {
            "command": "agent run",
            **prepare_profile_for_telemetry(profile),
            "agent_name_sha256": hash_value(agent_name),
            "debug": debug,
            "has_payload": payload is not None,
            "has_pagerduty_incident": bool(pagerduty_incident),
        }
    )
    plugin_manager = PluginManager(load_config(profile, create=False))
    data = ""
    if pagerduty_incident:
        if payload is not None or not sys.stdin.isatty():
            print("[red]Cannot pass --pagerduty-incident with --payload or stdin.[/red]")
            sys.exit(1)
        incident_id = pagerduty_incident
        if "/" in pagerduty_incident:
            incident_id = [x for x in pagerduty_incident.split("/") if x][-1]
        pd = cast("PagerDutyPlugin", plugin_manager.get_plugin("pagerduty"))
        incident = await pd.get_incident_by_id(incident_id)
        data = incident.model_dump_json()

    # Read data from stdin if it's being piped to us.
    if not data and not sys.stdin.isatty():
        if payload is not None:
            print("[red]Cannot pass a payload argument when piping data to stdin.[/red]")
            sys.exit(1)
        data = sys.stdin.read().strip()
    elif not data:
        # Otherwise, use the payload argument.
        data = payload

    if not data:
        print("[red]No payload provided.[/red]")
        print(
            "[bold]Pass an alert payload as an argument or pipe the payload data to stdin.[/bold]"
        )
        sys.exit(1)

    # Get the config directory and load the specific agent
    try:
        agent = load_agent(agent_name, profile)
    except FileNotFoundError as ex:
        print(f"[red]Agent {agent_name!r} not found at {str(ex.filename)!r}[/red]")
        print(f"[bold]Use 'unpage agent create {agent_name!r}' to create a new agent.[/bold]")
        sys.exit(1)

    # Run the analysis with the specific agent
    analysis_agent = AnalysisAgent(profile)
    try:
        result = await analysis_agent.acall(payload=data, agent=agent)
        print(result)
    except Exception as ex:
        print(f"[red]Analysis failed:[/red] {ex}")
        sys.exit(1)
    finally:
        if debug:
            print("\n\n===== DEBUG OUTPUT =====\n")
            analysis_agent.inspect_history(n=1000)
            print("\n===== END DEBUG OUTPUT =====\n\n")



import asyncio
import shutil
from typing import Annotated

from unpage.cli.graph._app import graph_app
from unpage.cli.graph._background import get_log_file
from unpage.cli.options import DEFAULT_PROFILE, ProfileParameter
from unpage.telemetry import client as telemetry
from unpage.telemetry import prepare_profile_for_telemetry


@graph_app.command
async def logs(
    *,
    profile: Annotated[str, ProfileParameter] = DEFAULT_PROFILE,
    follow: bool = False,
) -> None:
    """View graph build logs

    Parameters
    ----------
    profile
        The profile to use
    follow
        Follow log output
    """
    await telemetry.send_event(
        {
            "command": "graph logs",
            **prepare_profile_for_telemetry(profile),
            "follow": follow,
        }
    )
    log_file = get_log_file(profile)
    tail_cmd = shutil.which("tail")
    if not tail_cmd:
        print("'tail' command not found. Please install it.")
        return

    if not log_file.exists():
        print(f"No log file found for profile '{profile}'")
        print(f"Expected location: {log_file}")
        return

    if follow:
        print(f"Following logs for profile '{profile}' (Ctrl+C to stop)")
        print(f"Log file: {log_file}")

        try:
            proc = await asyncio.create_subprocess_shell(f"{tail_cmd} -f {log_file!s}")
            await proc.wait()
        except KeyboardInterrupt:
            print("\nStopped following logs")
    else:
        print(f"Recent logs for profile '{profile}':")
        print(f"Log file: {log_file}")

        proc = await asyncio.create_subprocess_shell(f"{tail_cmd} -n 50 {log_file!s}")
        await proc.wait()



from cyclopts import Parameter

DEFAULT_PROFILE = "default"

ProfileParameter = Parameter(
    name="--profile",
    help="Use profiles to manage multiple graphs",
    env_var="UNPAGE_PROFILE",
    show_env_var=True,
)



from typing import Annotated

from unpage.agent.utils import delete_agent
from unpage.cli.agent._app import agent_app
from unpage.cli.options import DEFAULT_PROFILE, ProfileParameter
from unpage.telemetry import client as telemetry
from unpage.telemetry import hash_value, prepare_profile_for_telemetry


@agent_app.command
async def delete(
    agent_name: str, /, *, profile: Annotated[str, ProfileParameter] = DEFAULT_PROFILE
) -> None:
    """Delete an agent.

    Parameters
    ----------
    agent_name
        The name of the agent to delete
    profile
        The profile to use
    """
    await telemetry.send_event(
        {
            "command": "agent delete",
            "agent_name_sha256": hash_value(agent_name),
            **prepare_profile_for_telemetry(profile),
        }
    )
    delete_agent(agent_name, profile)
